package com.diagbot.facade;

import com.alibaba.fastjson.JSONArray;
import com.diagbot.client.ChiefPresentSimilarityServiceClient;
import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.DrugBillNeoDTO;
import com.diagbot.dto.LisBillNeoDTO;
import com.diagbot.dto.PacsBillNeoDTO;
import com.diagbot.dto.PushDTO;
import com.diagbot.entity.node.*;
import com.diagbot.enums.StandConvertEnum;
import com.diagbot.model.ai.ModelAI;
import com.diagbot.repository.*;
import com.diagbot.util.ListUtil;
import com.diagbot.vo.BillNeoVO;
import com.diagbot.vo.PushNeoVO;
import com.diagbot.vo.StandConvert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import java.util.*;

/**
 * @Description: 图谱facade
 * @author: Mark
 * @time: 2018/8/6 9:11
 */
@Component
public class NeoFacade {

    @Autowired
    MedicineRepository medicineRepository;
    @Autowired
    PacsRepository pacsRepository;
    @Autowired
    LisRepository lisRepository;
    @Autowired
    DiseaseRepository diseaseRepository;
    @Autowired
    MedicineCodeRepository medicineCodeRepository;
    @Autowired
    ChiefPresentSimilarityServiceClient chiefPresentSimilarityServiceClient;
    @Autowired
    NeoFacade self;

    /**
     * 返回药品缓存信息
     *
     * @return
     */
    @Cacheable(value = "cache", key = "'drugCache'")
    public Map<String, List<String>> getDrugCache() {
        Map<String, List<String>> res = new HashMap<>();

        Iterator<Medicine_Code> iterator = medicineCodeRepository.findAll().iterator();
        Medicine_Code current;
        String med_code, med_type;
        List<OralMedicine> sets;

        while (iterator.hasNext()) {
            current = iterator.next();
            if (current.getMedicine_related_chem().size()>0) {
                sets = new ArrayList<>(current.getMedicine_related_chem());
                med_code = current.getName();
                med_type = sets.get(0).getName();
                if (res.get(med_type)==null) {
                    res.put(med_type, new ArrayList<>());
                }
                if (res.get(med_type).indexOf(med_code)==-1) {
                    res.get(med_type).add(med_code);
                }
            }
        }

        // TODO 测试数据
        res.put("泰舒达类", Arrays.asList("泰舒达"));
        res.put("青霉素类", Arrays.asList("青霉素"));
        return res;
    }


    /**
     * 返回文本相似度匹配需要的词性列表
     *
     * @return
     */
    @Cacheable(value = "cache", key = "'similarCache'")
    public Map<String, List<String>> getSimilarCache() {
        Map<String, List<String>> res = new HashMap<>();
        // 临床表现，诊断，化验，辅检，药品，查体，手术
        // TODO 测试数据
        res.put(StandConvertEnum.clinical.getName(), Arrays.asList("胸痛","咳嗽"));
        res.put(StandConvertEnum.disease.getName(), Arrays.asList("高血压", "糖尿病"));
        return res;
    }

    /**
     * 通过诊断反推
     *
     * @return
     */
    public Map<String, PushDTO> getInverse(PushNeoVO pushNeoVO) {
        Map<String, PushDTO> map = getDiag(pushNeoVO.getDiags());

        return map;
    }

    /**
     * 处理开单合理性图谱数据
     * @param billNeoVO
     * @return
     */
    public List<BillNeoDTO> getBillNeo(BillNeoVO billNeoVO) {

        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();

        billNeoDTOs.addAll(getDrugBill(billNeoVO.getDrugList()));
        billNeoDTOs.addAll(getPacsBill(billNeoVO.getPacsList()));
        billNeoDTOs.addAll(getLisBill(billNeoVO.getLisList()));

        for (BillNeoDTO billNeoDTO:billNeoDTOs) {
            System.out.println(billNeoDTO);
        }
        return billNeoDTOs;
    }



    /**
     * 处理处方开单合理性
     */
    public List<BillNeoDTO> getDrugBill(Map<String, String> medsterm) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        DrugBillNeoDTO drugBillNeoDTO;

        MedicineNode medicineNode = new MedicineNode();
        Medicine medicine;
        String medname;

        for (String term : medsterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            medname = medsterm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(medname);

            drugBillNeoDTO = new DrugBillNeoDTO();

            medicine = medicineRepository.findByNameIs(medname);

            if (medicine != null) {
                drugBillNeoDTO = medicineNode.MedicinetoDrugDTO(medicine);
            }

            billNeoDTO.setDrugBillNeoDTO(drugBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }


        return billNeoDTOs;
    }


    /**
     * 处理辅检开单合理性
     */
    public List<BillNeoDTO> getPacsBill(Map<String, String> pacsterm) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        PacsBillNeoDTO pacsBillNeoDTO;

        PACSNode pacsNode = new PACSNode();
        PACS pacs;
        String pacsname;

        for (String term : pacsterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            pacsname = pacsterm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(pacsname);

            pacsBillNeoDTO = new PacsBillNeoDTO();

            pacs = pacsRepository.findByNameIs(pacsname);

            if (pacs != null) {
                pacsBillNeoDTO = pacsNode.PacstoPACSDTO(pacs);
            }

            billNeoDTO.setPacsBillNeoDTO(pacsBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }


    /**
     * 处理化验开单合理性
     */
    public List<BillNeoDTO> getLisBill(Map<String, String> listerm) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        LisBillNeoDTO lisBillNeoDTO;

        LISNode lisNode = new LISNode();
        LIS lis;
        String lisname;

        for (String term : listerm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            lisname = listerm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(lisname);

            lisBillNeoDTO = new LisBillNeoDTO();

            lis = lisRepository.findByNameIs(lisname);

            if (lis != null) {
                lisBillNeoDTO = lisNode.ListoLISDTO(lis);
            }

            billNeoDTO.setLisBillNeoDTO(lisBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }


    public Map<String, PushDTO> getDiag(List<String> diaglist) {

        Map<String, PushDTO> pushDTOs = new HashMap<>();
        PushDTO pushDTO;

        DiseaseNode diseaseNode = new DiseaseNode();
        Disease disease;

        for (String term : diaglist) {

            disease = diseaseRepository.findByNameIs(term);

            if (disease != null) {
                pushDTO = diseaseNode.DiseasetoDiseaseDTO(disease);

                pushDTOs.put(term, pushDTO);
            }
        }

        return pushDTOs;
    }


    /**
     * 标准词转换
     *
     * @param standConvert
     *
     * @return  Map<String, Map<String, String>> -->Map<类型, Map<原始词, 标准词>>
     */
    public Map<String, Map<String, String>> standConvert(StandConvert standConvert) {
        Map<String, Map<String, String>> map = new LinkedHashMap<>();
        Map<String, List<String>> similarMap = self.getSimilarCache();
        /* 文本相似度模型 */
        ModelAI modelAI = new ModelAI();

        // 临床表现数据
        List<String> list = standConvert.getClinicalList();
        Map<String, String> clinicalMap = getStandConvertTypeMap(modelAI, list,
                similarMap.get(StandConvertEnum.clinical.getName()), "chief_present");
        map.put(StandConvertEnum.clinical.getName(), clinicalMap);

        // 诊断数据
        List<String> diag_list = standConvert.getDiaglList();
        Map<String, String> diseaseMap = getStandConvertTypeMap(modelAI, diag_list,
                similarMap.get(StandConvertEnum.disease.getName()), "diagnose");
        map.put(StandConvertEnum.disease.getName(), diseaseMap);

        // 手术数据
        List<String> operation_list = standConvert.getOperationList();
        Map<String, String> operationMap = getStandConvertTypeMap(modelAI, operation_list,
                similarMap.get(StandConvertEnum.operation.getName()), "operation");
        map.put(StandConvertEnum.operation.getName(), operationMap);

        return map;
    }

    /**
     * 获取指定类型的map数据
     *
     * @param modelAI
     * @param text 文本内容
     * @param cacheList 模型列表
     * @param modelName 模块名称
     * @return
     */
    public Map<String, String> getStandConvertTypeMap(ModelAI modelAI, List<String> text, List<String> cacheList, String modelName) {
        Map<String, String> map = new LinkedHashMap<>();
        if (ListUtil.isEmpty(cacheList)) {
            return map;
        }
        for (String s : text) {
            JSONArray jsonArray = modelAI.loadSimilarAI(s, cacheList,
                    false, modelName, chiefPresentSimilarityServiceClient);
            putMap(jsonArray, s, map);
        }
        return map;
    }

    /**
     * 提取方法
     * @param jsonArray
     * @param originName
     * @param map
     */
    public void putMap(JSONArray jsonArray, String originName, Map<String, String> map) {
        if (jsonArray.size() == 2) {
            double likeRate = jsonArray.getDoubleValue(1);
            // System.out.println(originName + " " + likeRate);
            if (likeRate > 0.9) {
                String standName = jsonArray.getString(0);
                map.put(originName, standName);
            } else {
                map.put(originName, originName);
            }
        }
    }

}
